;; -*- scheme -*-
(add-load-path "./threads")
(add-load-path "./time")
(import (srfi :64 testing)
	(srfi :39)
	(rnrs)
	(sagittarius)
	(sagittarius filewatch)
	(sagittarius threads)
	;; need shared-queue
	(util concurrent))

(test-begin "Sagittarius filewatch")

(test-assert make-filesystem-watcher)

(let ((w (make-filesystem-watcher)))
  (test-assert (filesystem-watcher? w))
  (test-assert (release-filesystem-watcher! w)))

(define-constant +file+ "watch")

(unless (file-exists? +file+)
  (call-with-output-file +file+ (lambda (out) (put-string out "created"))))

(define (run-tests)
;; NB: we can't test 'access since Windows Vista or later disabled 
;;     updating access timestamp by default.
(let* ((w (make-filesystem-watcher))
       (f (absolute-path +file+))
       (sq (make-shared-queue))
       (h (lambda (p e) (shared-queue-put! sq (cons p e)))))
  ;; add or remove
  (test-assert "add path" 
	       (filesystem-watcher? 
		(filesystem-watcher-add-path! w +file+ 'modify h)))
  (test-assert "remove path" 
	       (filesystem-watcher? (filesystem-watcher-remove-path! w +file+)))
  ;; add again
  (test-assert (filesystem-watcher? 
		(filesystem-watcher-add-path! w +file+ 'modify h)))
  (test-assert (filesystem-watcher? (filesystem-watcher-start-monitoring! w)))
  (test-error assertion-violation?
	      (filesystem-watcher-add-path! w +file+ 'access h))
  (test-error assertion-violation? (filesystem-watcher-remove-path! w +file+))
  (let loop ()
    (call-with-port (open-file-output-port +file+ (file-options no-fail)
					   (buffer-mode block)
					   (native-transcoder))
      (lambda (out) (put-string out "modify!")))
    ;; retry
    (when (shared-queue-empty? sq) (loop)))
  (test-equal "monitoring handler result" 
	      (cons f 'modified) (shared-queue-get! sq))
  (test-assert (filesystem-watcher-stop-monitoring! w))
  (test-assert (release-filesystem-watcher! w))
  )

(let* ((w (make-filesystem-watcher))
       (f (absolute-path +file+))
       (sq1 (make-shared-queue))
       (sq2 (make-shared-queue))
       (h (lambda (p e) (shared-queue-put! sq1 p)))
       (t1 (thread-start!
	    (make-thread
	     (lambda ()
	       (shared-queue-get! sq1)
	       (filesystem-watcher-stop-monitoring! w)
	       (shared-queue-put! sq2 'done!)))))
       (t2 (make-thread
	    (lambda ()
	      (let loop ()
		(let ((o (shared-queue-get! sq2 0 #f)))
		  (unless o
		    (call-with-port (open-file-output-port +file+
				      (file-options no-fail)
				      (buffer-mode block)
				      (native-transcoder))
		      (lambda (out) (put-string out "modify!")))
		    (loop))))))))
  ;; add or remove
  (test-assert "add path" 
	       (filesystem-watcher? 
		(filesystem-watcher-add-path! w +file+ 'modify h)))
  (thread-start! t2)
  (test-assert (filesystem-watcher?
		(filesystem-watcher-start-monitoring! w :background #f)))
  (thread-join! t1)
  (thread-join! t2)
  (test-assert (release-filesystem-watcher! w)))

;; finishing watcher from event handler
(let* ((w (make-filesystem-watcher))
       (f (absolute-path +file+))
       (sq1 (make-shared-queue))
       (h (lambda (p e)
	    (shared-queue-put! sq1 p)
	    (filesystem-watcher-stop-monitoring! w)))
       (t1 (make-thread
	    (lambda ()
	      (let loop ()
		(let ((o (shared-queue-get! sq1 0 #f)))
		  (unless o
		    (call-with-port (open-file-output-port +file+
				      (file-options no-fail)
				      (buffer-mode block)
				      (native-transcoder))
		      (lambda (out) (put-string out "modify!")))
		    (loop))))))))
  ;; add or remove
  (test-assert "add path" 
	       (filesystem-watcher? 
		(filesystem-watcher-add-path! w +file+ 'modify h)))
  (thread-start! t1)
  (test-assert (filesystem-watcher?
		(filesystem-watcher-start-monitoring! w :background #f)))
  (thread-join! t1)
  (test-assert (release-filesystem-watcher! w)))
)

(run-tests)

(test-end)
